#!/usr/bin/python
############################################################
# first draft for a drug interactions XMLRPC service server
#
# (c) by Dr Horst Herb, released under the GPL free software
# license, details see http://www.gnu.org
############################################################

import string, SimpleXMLRPCServer
from pyPgSQL import PgSQL as dbapi


def queryresult2rowdict(queryresult):
	rows = []
	for row in queryresult:
		d = {}
		for f in row.keys():
			d[f]=row[f]
		rows.append(d)
	return rows


def count(con, what):
	cur = con.cursor()
	cur.execute("insert into counter(what) values(%d)" % what)
	con.commit()

def getCanDrugCode(cur,drugCode):
        retCode = "0"
        sql = "select import_code from tags where tagged_id = '%s' " % drugCode
        cur.execute(sql)
        res = cur.fetchall()
        for code in res:
             retCode = code[0]
        return retCode


class InteractionInfo:
	"""This class allows to search a PostgreSQL database for drug names, ATC codes, and
	drug-drug interaction information"""

	def __init__(self, database="drugref", user=None, password=None):
		"""works only if the Postgres server is local.
		Parameters are database name, username and password"""
		self._db = database
		self._user = user
		self._pwd = password

	def atc(self, drug):
		"""returns all matching ATC codes for a given (fraction of) a drug name.
		Search is case insensitive"""
		con = dbapi.connect(database=self._db)
		#count(con, 1)
		cur = con.cursor()
		udrug = string.upper(drug)
		query = "select code, text from atc where text ilike '%s%%'" % udrug
		print query
		cur.execute(query)
		res = cur.fetchall()
		if len(res)>0:
			return queryresult2rowdict(res)
		else:
			return [{'code':'0', 'text':'None found'}]

	def atcFromBrand(self, drug):
                """returns all matching ATC codes for a given (fraction of) a drug name.
                Search is case insensitive"""
                con = dbapi.connect(database=self._db)
                #count(con, 1)
                cur = con.cursor()
                udrug = string.upper(drug)
                #query = "select code, text from atc where text like '%s%%'" % udrug
                query = "select atc.atccode as code, pm.brandname as text from link_product_manufacturer pm, product p, generic_drug_name g, link_drug_atc atc where pm.id_product = p.id and p.id_drug = g.id_drug  and g.id_drug = atc.id_drug and pm.brandname ilike  '%s%%'" % udrug
                print query
                cur.execute(query)
                res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'code':'0', 'text':'None found'}]
                                                                                                                                                                                                     
                                                                                                                                                                                                            
	def atc2text(self, code):
		"""returns the English name of a drug that matches the stated ATC code"""
		con = dbapi.connect(database=self._db)
		#count(con, 2)
		cur = con.cursor()
		ucode = string.upper(code)
		query = "select code, text from atc where code like '%s%%'" % ucode
		print query
		cur.execute(query)
		res = cur.fetchall()
		if len(res) > 0:
			return queryresult2rowdict(res)
		else:
			return [{'code':'0', 'text':'None found'}]


	def interaction(self, atclist, minimum_significance=1):
		"""returns a list of drug-drug interactions as list of "dicts"
		atclist : list of ATC codes
		minimum_significance: interactions below the stated significance level will be ignored"""
		con = dbapi.connect(database=self._db)
		count(con, 3)
		cur = con.cursor()
		results = []
		for drug in atclist:
			for affected_drug in atclist:
				if drug is affected_drug:
					continue
				udrug = string.upper(drug)
				uaffected_drug = string.upper(affected_drug)
				query = "select drug, effect, affected_drug, significance, evidence, reference from simple_interactions where drug = '%s' and affected_drug = '%s' and significance >= %d" % (udrug, uaffected_drug, minimum_significance)
				print query
				cur.execute(query)
				result = cur.fetchall()
				if len(result)>0:
					results.extend(queryresult2rowdict(result))
		return results

	def list_search_element(self, searchStr):
                """
		returns all matching search element names , ids and categories for the given searchString.
                Search is case insensitive"""
                con = dbapi.connect(database=self._db)
                #count(con, 1)
                cur = con.cursor()
                """udrug = string.upper(sea)"""
                query = "select distinct  d.id, d.category, dn.name from drug_element d,drug_element_name dn  where d.id = dn.id_drug_element and  dn.name ilike '%s%%' order by dn.name" % searchStr
                print query
                cur.execute(query)
                res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'id':'0','category':'','name':'None found'}]

                   

	
	def list_brands_from_element(self,drug_id):

		con = dbapi.connect(database=self._db)
                cur = con.cursor()
                
                query = "select d.id_drug_element as id,CAST(13 as integer) as category, d.name from drug_element_name d, link_drug_class l where l.id_drug = d.id_drug_element and l.id_class = '%s' order by d.name " % drug_id
                 
                print query
                cur.execute(query)
                res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'id':'0','category':'','name':'None found'}]


	def list_search_element_select_categories(self, searchStr,categories):
                con = dbapi.connect(database=self._db)
                cur = con.cursor()
                s = " and ( "
		catCount = 0
                for cate in categories:
			catCount = catCount +1
			if (catCount > 1 ): 
                        	s =  "%s  or  d.category = '%s' " %  (s,cate)
			else: 
				s = "%s d.category = '%s' " % (s,cate)
                if (catCount == 0):
        		s = ""
		else:
			s = "%s ) " % s
		                    
	                                                                                                                                 
                query = "select distinct  d.id, d.category, dn.name from drug_element d,drug_element_name dn  where d.id = dn.id_drug_element and  dn.name ilike '%s%%' %s order by d.category, dn.name" % (searchStr,s)
                print query
                cur.execute(query)
                res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'id':'0','category':'','name':'None found'}]
                                                                                                                                                             
        
	def get_drug(self, pKey, html):
		"""
		highest level API function to retrieve a complete drug monograph (except for packaging, pricing and subsidy information)
        	struct with at least the following keys:
		name: string. International nonproprietary name (INPN) of this drug (=”generic”)
		atc: string. ATC code
		essential: True if this drug is on the WHO essential drug list
		product: string. If this drug is not a generic, the product brand name is listed under this key, else this key is not available
		action: string. Description of mode of action.
		indications: array of structs. Each struct has “indication” as key, and a struct as value containing the following keys:
			- code : integer. Drugref condition code primary key
			- firstline : boolean. True if for this indication this drug is considered a first line treatment.
			- comment : string
		contraindications: array of structs. Each struct has “contraindication” as key, and a struct as value containing the following keys:
			- code : integer. Drugref condition code primary key.
			- severity : integer (1-3, 3 being absolute contraindication)
			- comment : string
		practice_points: array of strings
		paediatric_use:  string. Describing special considerations in paediatric use
		pregnancy_use: struct with following keys:
			- code : character. ADEC category
			- comment : string
		lactation_use: struct with following keys:
			* code : integer. 1=compatible, 2=restricted, 3=dangerous
			* comment : string
		renal_impairment: struct with following keys:
			- code : integer. 1=compatible, 2=restricted, 3=dangerous
			- comment : string
		hepatic_impairment: struct with following keys:
			- code : integer. 1=compatible, 2=restricted, 3=dangerous
			- comment : string
		common_adverse_effects: array of strings
		rare_adverse_effects: array of strings
		dosage: array of structs with following keys:
			text: string. If this key is available, only free text dosage information is available, as described in this string.
			* indication : integer. Drugref condition code primary key. 0 is wild card, indicating general dosage recommendation.
    			* units: string. SI unit for this dosage
    			* calculation_base_units: integer. 0=not applicable, 1=age in months, 2=age in years, 3=kg body weight, 4=cm2 body surface
    			* calculation_base : real. Meaning depends on calculation_base_units
    			* starting_range : real. Usual mimimal recommended dosage
    			* upper_range : real. Usual maximal recommended dosage
    			* frequency_units : integer. 0=not applicable, 1=seconds, 2=minutes, 3=hours, 4=days, 5=weeks, 6=months, 7=years
    			* frequency : integer. How often this drug should be administered
    			* duration_units: integer. Same as frequency_units, additional value 8='times'
    			* duration_minimum ; integer. How long a usual course of this drug should be given. -1 is “permanent”, -2=”p.r.n.”
    			* duration_maximum : integer. -1 is “permanent”, “-2”=”p.r.n.”
    			* constrained: boolean. If true, no automazied dosage suggestion must be generated, prescriber must read comment. (e.g. dosage per body surface etc.)
    			* comment
		"""
		con = dbapi.connect(database=self._db)
		cur = con.cursor()
		productId = ""	
	
        	name_query = "select d.name as name from link_drug_class l, drug_element_name d, drug_element e where l.id_drug = '%s' and l.id_class = d.id_drug_element and e.id = d.id_drug_element and ( category = 11 or category = 12 )" % (pKey)
        	atc_query  = "select atccode from link_drug_atc where id_drug = '%s'" % (pKey)
        	essential_query = ""  
		product_query = "select name from drug_element_name where id_drug_element = '%s'" % (pKey)
        	action_query = ""
		indications_query = "" 
        	contraindications_query = ""
        	practice_points_query = "" 
        	paediatric_use_query = ""
        	pregnancy_use_query = ""
        	lactation_use_query = ""
        	renal_impairment_query = ""
        	hepatic_impairment_query = ""
        	common_adverse_effects_query = ""
        	rare_adverse_effects_query = ""
        	dosage_query = ""
		
		regional_identifier_query = "select id,comment from product where id_drug = '%s'" % (pKey)

		component_query = "select  n.name, pc.strength, u.unit from link_product_component pc, drug_element_name n, drug_units u  where id_product = '%s' and pc.id_component = n.id_drug_element and pc.id_unit = u.id" % (productId)
				

		name = ""
                atc  = ""
                essential = ""
                product = ""
                action = ""
                indications = ""
                contraindications = ""
                practice_points = ""
                paediatric_use = ""
                pregnancy_use = ""
                lactation_use = ""
                renal_impairment = ""
                hepatic_impairment = ""
                common_adverse_effects = ""
                rare_adverse_effects = ""
                dosage = ""
                regional_identifier = "" 
		components = []

		cur.execute(name_query)
                res = cur.fetchall()
		if len(res)>0:
                        for code in res:
             			name = code[0]
		cur.execute(atc_query)
                res = cur.fetchall()
                if len(res)>0:
                        for code in res:
                                atc = code[0]
		cur.execute(product_query)
                res = cur.fetchall()
                if len(res)>0:
                        for code in res:
                                product = code[0]

		cur.execute(regional_identifier_query)
                res = cur.fetchall()
                if len(res)>0:
                        for code in res:
				productId = code[0]
                                regional_identifier = code[1]

		component_query = "select  n.name, pc.strength, u.unit from link_product_component pc, drug_element_name n, drug_units u  where id_product ='%s' and pc.id_component = n.id_drug_element and pc.id_unit = u.id" % (productId)

		cur.execute(component_query)
                res = cur.fetchall()
                if len(res)>0:
                        for code in res:
				ha = {}
				ha['name'] = code[0]
                                ha['strength'] = code[1]
                                ha['unit'] = code[2]
				components.append(ha)

 		
		returnRows = []
		returnVal = {}
		returnVal['name'] = name
		returnVal['atc']  = atc
		returnVal['product'] = product
		returnVal['regional_identifier'] = regional_identifier
		returnVal['components'] = components
		returnRows.append(returnVal)
		return returnRows

	def get_generic_name(self, drug_id):
                con = dbapi.connect(database=self._db)
                cur = con.cursor()

                query = "select  l.id_class as id_class, d.name as name from link_drug_class l, drug_element_name d, drug_element e where l.id_class = d.id_drug_element and e.id = d.id_drug_element and ( category = 11 or category = 12 ) and l.id_drug = '%s' order by l.id_class" % drug_id
         
                print query
                cur.execute(query)
                res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'id_class':'0','name':'None found'}]



	def  getTestForDin(self,din):
		con = dbapi.connect(database=self._db)
		cur = con.cursor()

		query = "select drug_code from cd_drug_product where drug_identification_number = '%s'" % din
		print query
		cur.execute(query)
		res = cur.fetchall()
		if not res or cur.rowcount < 1:
			query2 = " select drug_code from cd_inactive_products  where drug_identification_number = '%s'" % din
			cur.execute(query2)
			ina = cur.fetchall()
			if not ina or cur.rowcount < 1:
				return [{'found':'false'}]
			else:
				return [{'found':'true'}]
				
		else:
			return [{'found':'true'}]



	def suggestAlias(self, alias, aliasComment, id, name, provider):
		fout = file("~/suggestedAlias.txt", 'a')
		lineToWrite = " id = %s , alias = %s , name = %s , provider = %s , aliasComment = %s " % (id,alias,name,provider,aliasComment)
		fout.write(lineToWrite)
    		fout.close()
		
	
	def list_drug_class(self, Dclass):
                con = dbapi.connect(database=self._db)
                cur = con.cursor()
                whereString = " and ( "
                catCount = 0
                for item in Dclass: 
                        catCount = catCount + 1
                        if (catCount > 1 ):
                                whereString =  "%s  or  l.id_drug = '%s' " %  (whereString,item)
                        else:
                                whereString = "%s l.id_drug = '%s' " % (whereString,item)
                if (catCount == 0):
                        whereString = ""
                else:
                        whereString = " %s ) " % whereString

                query = "select l.id_drug as id_drug, l.id_class as id_class, d.name as name from link_drug_class l, drug_element_name d, drug_element e where l.id_class = d.id_drug_element and e.id = d.id_drug_element and category = 10  %s order by l.id_class" % whereString

                print query
                cur.execute(query)
                res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'id_drug':'0','id_class':'0','name':'None found'}]


        def getDrugInfoPage(self,drug_id):

		con = dbapi.connect(database=self._db)
                cur = con.cursor()
                query = "select hyper_link from drug_resource where id_drug = '%s' " % drug_id
                                                                                                                               
                print query
                cur.execute(query)
                res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'hyper_link':'http://oscarresource.org'}]

	def getGenericNamefromId(self,drug_id):
                                                                                                                                                             
                con = dbapi.connect(database=self._db)
                cur = con.cursor()
                                                                                                                                                             
                query = "select d.name from drug_element_name d, link_drug_class l where l.id_drug = d.id_drug_element and l.id_class = '%s' " % drug_id
                                                                                                                                                             
                print query
                cur.execute(query)
                res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'id':'0','category':'','name':'None found'}]

	def getCanDrugCode(cur,drugCode):
		retCode = "0"
		sql = "select import_code from tags where tagged_id = '%s' " % drugCode
		cur.execute(sql)
                res = cur.fetchall()
		for code in res:
			retCode = code[0]
		return retCode

	def getAISameByDrugCode(self,drugCode):
		con = dbapi.connect(database=self._db)
                cur = con.cursor()
		aiGroupNo = "0"
		drug = getCanDrugCode(cur,drugCode)
                sql = "select ai_group_no from CD_drug_product  where drug_code = '%s' " % drug 
                cur.execute(sql)
		rs = cur.fetchall()	
		
                for ai in rs:                    
                    aiGroupNo = ai['ai_group_no']                                        
                     
                sql2 = "select drug_code, brand_name from CD_drug_product where ai_group_no like '%s' " % aiGroupNo
                print sql2
                cur.execute(sql2)
		res = cur.fetchall()
		if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'drug_code':'None found','brand_name':'None found'}]

	def getFormFromDrugCode(self,drugCode):

		con = dbapi.connect(database=self._db)
                cur = con.cursor()
		sql = "SELECT pharmaceutical_CD_form as form FROM CD_form where drug_code = '%s' " % getCanDrugCode(cur,drugCode)       
                print sql
                cur.execute(sql)
		res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'form':'None found'}]

       	def getDistinctForms(self):
                                                                                                                            
                con = dbapi.connect(database=self._db)
                cur = con.cursor()
                sql = "SELECT distinct  pharmaceutical_CD_form as form FROM cd_form " 
                print sql
                cur.execute(sql)
                res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'form':'None found'}]
 
        """                                              """
        """
	select name from drug_element_name where id_drug_element in (select id_component from link_product_component where id_product = (select distinct id from drug_root_item  where id_drug = '90420' ));

	"""	

	def getComponents(self,drugId):
                                                                                                                                                             
                con = dbapi.connect(database=self._db)
                cur = con.cursor()
		sql = "select name from drug_element_name where id_drug_element in (select id_component from link_product_component where id_product = (select distinct id from drug_root_item  where id_drug = '%s' )) " %  drugId
                print sql
                cur.execute(sql)
                res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'name':'None found'}]


        def getBrandNameFromDIN(self,din):

                con = dbapi.connect(database=self._db)
                cur = con.cursor()
                sql = "select brand_name from cd_drug_product where drug_identification_number = '%s' " %  din
                print sql
                cur.execute(sql)
                res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'brand_name':'None found'}]


	
        def getRouteFromDrugCode(self,drugCode):

                con = dbapi.connect(database=self._db)
                cur = con.cursor()
                sql = "SELECT route_of_administration as route FROM cd_route where drug_code = '%s' " % getCanDrugCode(cur,drugCode)
                print sql
                cur.execute(sql)
                res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'route':'None found'}]



	def getStrengths(self,drugCode):
		con = dbapi.connect(database=self._db)
		cur = con.cursor()
		sql = "select ingredient, strength, strength_unit from cd_active_ingredients where drug_code = '%s' " % getCanDrugCode(cur,drugCode)
                cur.execute(sql);
                res = cur.fetchall();
                if len(res)>0:
                        return queryresult2rowdict(res)                                                                                                                           
                else: 
                        return [{'ingredient':'None found','strength':'0','strength_unit':'None found'}]


	"""                                            """
	def getProductData(self,drug_id):
		con = dbapi.connect(database=self._db)
                cur = con.cursor()
                query = "select p.id_drug as id, f.description as form ,r.description as route ,u.unit as unit,p.amount as amount from product p, drug_formulations f, drug_units u, drug_routes r where f.id = p.id_formulation and  u.id = p.id_packing_unit and r.id = p.id_route and  id_drug = '%s' " %  drug_id
                
                cur.execute(query)
                res = cur.fetchall()
                if len(res)>0:
                        return queryresult2rowdict(res)
                else:
                        return [{'id':'0','form':'','route':'','unit':'','amount':''}]


#	def drug2atclist(self, drug):
#		atcs = self.atc(drug)
#		result = []
#		for code in atcs:
#			result.append(code['code'])
#		return result



#	def druglist2atclist(self, druglist):
#		atclist = []
#		for drug in druglist:
#			atclist.extend(drug2atclist(drug))


#	def interaction_by_drugnames(self, druglist, minimum_significance=1):
#		
#		atclist = druglist2atclist(druglist)
#		return(interaction(atclist, minimum_significance)
 



if __name__=='__main__' :
	import sys

	#parse the command line:
	try:
		server_url = sys.argv[1]
	except:
		server_url = "localhost"
	try:
		server_port = sys.argv[2]
	except:
		server_port = 8001

	print "Serving at port", server_port, server_url
	server=SimpleXMLRPCServer.SimpleXMLRPCServer((server_url, server_port))
	server.register_instance(InteractionInfo())
	try:
		server.serve_forever()
	except:
		#exit gracefully, release port
		del(server)


